﻿Imports System

''' <summary>
'''  Wrapped einen frei definierbaren Initialisierung- und einen 
'''   Aufräum-Delegaten, sodass ein Aufruf von "Using DisposeLevel.Enter()" 
'''   sicher initialisiert und aufräumt.
'''  Bei Verschachtelung wird nur beim Betreten/Verlassen des äußeren 
'''   Using-Blocks initialisiert/aufgeräumt 
''' </summary>
Public Class DisposeLevel : Implements IDisposable

    ' Dispose() wird hier "missbraucht", um den Level runterzuzählen, und bei 0
    '  die Aufräum-Aktion aufzurufen. IDisposable ermöglicht die Verwendung der 
    '  sehr sicheren Using-Block-Syntax.

    Protected _enter As Action, _exit As Action
    Protected _level As Integer = 0

    Public Sub New()
    End Sub
    Public Sub New(ByVal enter As Action, ByVal [exit] As Action)
        Add(enter, [exit])
    End Sub
    Private Sub EnterCore()
        _level += 1
        If _level = 1 Then _enter()
    End Sub
    Public Function Enter() As IDisposable
        EnterCore()
        Return Me
    End Function
    Private Sub Dispose() Implements IDisposable.Dispose
        _level -= 1
        Select Case _level
            Case -1
                Throw Me.Exception("cannot set Level < 0!")
            Case 0
                _exit()
        End Select
    End Sub
    ''' <summary>
    ''' Hinzufügen weiterer DisposeLevel lässt baumartige Ablauf-Strukturen
    '''  entstehen.
    ''' </summary>
    Public Sub Add(ByVal child As DisposeLevel)
        Add(AddressOf child.EnterCore, AddressOf child.Dispose)
    End Sub
    Public Sub Add(ByVal enter As Action, ByVal [exit] As Action)
        _enter = If(_enter Is Nothing, enter, _enter.CombineWith(enter))
        _exit = If(_exit Is Nothing, [exit], [exit].CombineWith(_exit))
    End Sub
End Class
